@@ -1,4 +1,5 @@
 require 'beaker-rspec'
+require 'beaker/puppet_install_helper'
 
 def iptables_flush_all_tables
   ['filter', 'nat', 'mangle', 'raw'].each do |t|
@@ -12,17 +13,16 @@ def ip6tables_flush_all_tables
   end
 end
 
-unless ENV['RS_PROVISION'] == 'no' or ENV['BEAKER_provision'] == 'no'
-  if hosts.first.is_pe?
-    install_pe
+def do_catch_changes
+  if default['platform'] =~ /el-5/
+    return false
   else
-    install_puppet
-  end
-  hosts.each do |host|
-    on host, "mkdir -p #{host['distmoduledir']}"
+    return true
   end
 end
 
+run_puppet_install_helper
+
 UNSUPPORTED_PLATFORMS = ['windows','Solaris','Darwin']
 
 RSpec.configure do |c|
@@ -35,10 +35,9 @@ RSpec.configure do |c|
   # Configure all nodes in nodeset
   c.before :suite do
     # Install module and dependencies
-    puppet_module_install(:source => proj_root, :module_name => 'firewall')
     hosts.each do |host|
-      shell('/bin/touch /etc/puppet/hiera.yaml')
-      shell('puppet module install puppetlabs-stdlib --version 3.2.0', { :acceptable_exit_codes => [0,1] })
+      copy_module_to(host, :source => proj_root, :module_name => 'firewall')
+      on host, puppet('module install puppetlabs-stdlib --version 3.2.0'), { :acceptable_exit_codes => [0,1] }
     end
   end
 end
