@@ -11,6 +11,16 @@ describe 'firewall', :type => :class do
     it { expect { should contain_class('firewall::linux') }.to raise_error(Puppet::Error) }
   end
 
+  context 'kernel => SunOS' do
+    let(:facts) {{ :kernel => 'SunOS' }}
+    it { expect { should contain_class('firewall::linux') }.to raise_error(Puppet::Error) }
+  end
+
+  context 'kernel => Darwin' do
+    let(:facts) {{ :kernel => 'Darwin' }}
+    it { expect { should contain_class('firewall::linux') }.to raise_error(Puppet::Error) }
+  end
+
   context 'ensure => stopped' do
     let(:facts) {{ :kernel => 'Linux' }}
     let(:params) {{ :ensure => 'stopped' }}
